@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004-2008 Zope Corporation and Contributors.
+# Copyright (c) 2004-2008 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -231,7 +231,6 @@ class Runner(object):
                 break
 
         if should_resume:
-            setup_layers = None
             if layers_to_run:
                 self.ran += resume_tests(
                     self.script_parts, self.options, self.features,
@@ -601,15 +600,17 @@ def tear_down_unneeded(options, needed, setup_layers, optional=False):
         output.start_tear_down(name_from_layer(l))
         t = time.time()
         try:
-            if hasattr(l, 'tearDown'):
-                l.tearDown()
-        except NotImplementedError:
-            output.tear_down_not_supported()
-            if not optional:
-                raise CanNotTearDown(l)
-        else:
-            output.stop_tear_down(time.time() - t)
-        del setup_layers[l]
+            try:
+                if hasattr(l, 'tearDown'):
+                    l.tearDown()
+            except NotImplementedError:
+                output.tear_down_not_supported()
+                if not optional:
+                    raise CanNotTearDown(l)
+            else:
+                output.stop_tear_down(time.time() - t)
+        finally:
+            del setup_layers[l]
 
 
 cant_pm_in_subprocess_message = """
@@ -711,6 +712,8 @@ class TestResult(unittest.TestResult):
                                                       " as a subprocess!")
             else:
                 zope.testing.testrunner.debug.post_mortem(exc_info)
+        elif self.options.stop_on_error:
+            self.stop()
 
     def addFailure(self, test, exc_info):
         self.options.output.test_failure(test, time.time() - self._start_time,
@@ -722,6 +725,8 @@ class TestResult(unittest.TestResult):
             # XXX: mgedmin: why isn't there a resume_layer check here like
             # in addError?
             zope.testing.testrunner.debug.post_mortem(exc_info)
+        elif self.options.stop_on_error:
+            self.stop()
 
     def stopTest(self, test):
         self.testTearDown()
